package ovh.herisson.Clyde.DTO.ScientificPublications;

/******************************************************
 * @file ResearchDTO.java
 * @author Bartha Maxime
 * @scope Publications Scientifiques
 *
 * Research format to return to front (without author's password)
 ******************************************************/

import lombok.Data;
import ovh.herisson.Clyde.Tables.ScientificPublications.Access;
import ovh.herisson.Clyde.Tables.ScientificPublications.PaperType;
import ovh.herisson.Clyde.Tables.ScientificPublications.Research;
import ovh.herisson.Clyde.Tables.ScientificPublications.Researcher;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

@Data
public class ResearchDTO {

    private long id;
    private String title;
    private ResearcherDTO researcher;
    private final Set<ResearcherDTO> coAuthors;
    private Date releaseDate;
    private PaperType paperType;
    private String pdfLocation;
    private String bibTexLocation;
    private String language;
    private Access access;
    private String domain;
    private String summary;
    private long views;

    private ResearchDTO(String title, ResearcherDTO researcherDTO, Date releaseDate, PaperType paperType, String pdfLocation, String language, Access access, String domain, String bibTexLocation, String summary, Set<Researcher> coAuthors, long id, long views) {
        this.title = title;
        this.researcher = researcherDTO;
        this.releaseDate = releaseDate;
        this.paperType = paperType;
        this.pdfLocation = pdfLocation;
        this.language = language;
        this.access = access;
        this.domain = domain;
        this.summary = summary;
        this.id = id;
        this.bibTexLocation = bibTexLocation;
        this.coAuthors = new HashSet<>();
        for (Researcher coAuthor: coAuthors) {
            this.coAuthors.add(ResearcherDTO.construct(coAuthor));
        }
        this.views = views;
    }


    public static ResearchDTO construct(Research research){
        return new ResearchDTO(research.getTitle(), ResearcherDTO.construct(research.getAuthor()), research.getReleaseDate(),
                research.getPaperType(),research.getPdfLocation(),research.getLanguage(),research.getAccess(),
                research.getDomain(),research.getBibTexLocation(), research.getSummary(), research.getCoAuthors(),research.getId(), research.getViews());
    }
}
