package ovh.herisson.Clyde.EndPoints.Inscription;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Repositories.Inscription.MinervalRepository;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Tables.Inscription.Minerval;
import ovh.herisson.Clyde.Tables.Role;

import java.util.*;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
public class MinervalController {
    private final AuthenticatorService authServ;
    private final MinervalRepository mr;

    public MinervalController(AuthenticatorService authServ, MinervalRepository mr) {
        this.authServ = authServ;
        this.mr = mr;
    }

    //A new minerval entry is posted when the inscription service accept a registration request
    @PostMapping("/minerval/{studentRegNo}")
    public ResponseEntity<Object> postMinerval(@RequestHeader("Authorization") String token, @PathVariable long studentRegNo){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Secretary,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        Calendar c = new GregorianCalendar();

        mr.save(new Minerval(studentRegNo, 0, 835, c.get(Calendar.YEAR)));
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping("/minerval/{studentRegNo}")
    public ResponseEntity<Minerval> getCurrentMinervalbyRegNo(@PathVariable long studentRegNo){
        ArrayList<Minerval> mlist = mr.getMinervalsByStudentRegNoOrderByYearDesc(studentRegNo);

        //The list is ordered by year in descending order then the index 0 contains the actual minerval (for this year)
        Minerval m = mlist.get(0);
        return new ResponseEntity<>(m, HttpStatus.OK);
    }

    @PatchMapping("/minerval")
    public ResponseEntity<Object> updateMinerval(@RequestBody Minerval updatedMinerval){
        Minerval minerval = mr.findById(updatedMinerval.getId());

        minerval.setPaidAmount(updatedMinerval.getPaidAmount());
        minerval.setToPay(updatedMinerval.getToPay());
        mr.save(minerval);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}
