package ovh.herisson.Clyde.EndPoints.Inscription;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Repositories.CourseRepository;
import ovh.herisson.Clyde.Repositories.CurriculumRepository;
import ovh.herisson.Clyde.Repositories.Inscription.*;
import ovh.herisson.Clyde.Repositories.UserCurriculumRepository;
import ovh.herisson.Clyde.Repositories.UserRepository;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Services.TokenService;
import ovh.herisson.Clyde.Services.UserService;
import ovh.herisson.Clyde.Tables.*;
import ovh.herisson.Clyde.Tables.Inscription.ExemptionsRequest;
import ovh.herisson.Clyde.Tables.Inscription.Minerval;
import ovh.herisson.Clyde.Tables.Inscription.ScholarshipRequest;
import ovh.herisson.Clyde.Tables.Inscription.UnregisterRequest;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
public class RequestsController {

    public final TokenService tokenService;
    public final ExemptionsRequestRepository err;
    public final ScholarshipRequestRepository srr;
    public final UserRepository userRepository;
    public final AuthenticatorService authServ;
    public final UnregisterRequestRepository unregisterRequestRepository;
    public final CourseRepository courseRepository;
    public final UserService userService;
    public final UserCurriculumRepository userCurriculumRepository;
    public final CurriculumRepository curriculumRepository;
    public final MinervalRepository minervalRepository;
    public final ChangeCurriculumRequestRepository changeCurriculumRequestRepository;

    public RequestsController(TokenService tokenService, ExemptionsRequestRepository err, ScholarshipRequestRepository srr, UserRepository userRepository, AuthenticatorService authServ, UnregisterRequestRepository unregisterRequestRepository, CourseRepository courseRepository, UserService userService, UserCurriculumRepository userCurriculumRepository, CurriculumRepository curriculumRepository, MinervalRepository minervalRepository, ChangeCurriculumRequestRepository changeCurriculumRequestRepository) {
        this.tokenService = tokenService;
        this.err = err;
        this.srr = srr;
        this.userRepository = userRepository;
        this.authServ = authServ;
        this.unregisterRequestRepository = unregisterRequestRepository;
        this.courseRepository = courseRepository;
        this.userService = userService;
        this.userCurriculumRepository = userCurriculumRepository;
        this.curriculumRepository = curriculumRepository;
        this.minervalRepository = minervalRepository;
        this.changeCurriculumRequestRepository = changeCurriculumRequestRepository;
    }

    @PostMapping(value="/exemptionreq")
    public ResponseEntity<String> createExemptionReq(@RequestBody Map<String, Object> exemptionsRequestInfo){
        User user = userRepository.findById((Integer) exemptionsRequestInfo.get("userRegNo"));
        Course course = courseRepository.findById((Integer) exemptionsRequestInfo.get("courseId"));

        ExemptionsRequest exemptionsRequest = new ExemptionsRequest(user, course, (String) exemptionsRequestInfo.get("justifDocument"), RequestState.Pending, new Date());

        err.save(exemptionsRequest);

        return new ResponseEntity<>(HttpStatus.CREATED);
    }

    @PostMapping(value="/scholarshipreq")
    public ResponseEntity<String> createScholarshipReq(@RequestBody Map<String, Object> scholarshipRequestInfo){
        User user = userRepository.findById((Integer)scholarshipRequestInfo.get("userId"));
        ScholarshipRequest toCreate = new ScholarshipRequest(user, RequestState.Pending, 0, new Date(), (String) scholarshipRequestInfo.get("taxDocUrl"), (String) scholarshipRequestInfo.get("residencyDocUrl"));

        srr.save(toCreate);

        return new ResponseEntity<>(HttpStatus.CREATED);
    }

    //Get all the exemptions Request
    @GetMapping(value = "/exemptionsreq")
    public ResponseEntity<ArrayList<ExemptionsRequest>> getAllExemptionsRequests(@RequestHeader("Authorization") String token){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService, Role.Teacher},token))
            return new UnauthorizedResponse<>(null);

        ArrayList<ExemptionsRequest> toReturn = new ArrayList<>();

        err.findAll().forEach(toReturn::add);

        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }

    @GetMapping(value = "/exemptionsreq/{id}")
    public ResponseEntity<ExemptionsRequest> getExemptionRequestbyId(@RequestHeader("Authorization") String token, @PathVariable long id){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Teacher,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        ExemptionsRequest exemptionsRequest = err.findById(id);

        return new ResponseEntity<>(exemptionsRequest, HttpStatus.OK);
    }

    @PatchMapping(value = "/exemptionsreq/{id}/{newstate}")
    public ResponseEntity<String> changeExemptionReqState(@RequestHeader("Authorization") String token, @PathVariable long id, @PathVariable RequestState newstate){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Teacher},token))
            return new UnauthorizedResponse<>(null);

        ExemptionsRequest exemptionsRequest = err.findById(id);

        if (exemptionsRequest.getState() == RequestState.Accepted){
            return new ResponseEntity<>(HttpStatus.OK);
        }

        exemptionsRequest.setState(newstate);
        err.save(exemptionsRequest);

        return new ResponseEntity<>(HttpStatus.OK);
    }

    //Get all the scholarships requests
    @GetMapping(value = "/scholarshipreq")
    public ResponseEntity<ArrayList<ScholarshipRequest>> getAllScholarshipRequests(@RequestHeader("Authorization") String token){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Secretary,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        ArrayList<ScholarshipRequest> toReturn = new ArrayList<>();

        srr.findAll().forEach(toReturn::add);

        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }

    @PostMapping(value = "/unregister")
    public ResponseEntity<String> postUnregReq(@RequestBody Map<String,Object> uninscr){
        User u = userRepository.findById((int) uninscr.get("userId"));
        Curriculum c;

        if (uninscr.get("curriculumId") == null){
            c = null;
        }else{
            c = curriculumRepository.findById((Integer) uninscr.get("curriculumId"));
        }

        UnregisterRequest ur = new UnregisterRequest(RequestState.Pending, (String) uninscr.get("reason"), new Date(), u.getRegNo(), u.getFirstName(), u.getLastName(), u.getEmail(), c);
        unregisterRequestRepository.save(ur);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PatchMapping(value = "/scholarshipreq/")
    public ResponseEntity<String> editScholReq(@RequestHeader("Authorization") String token, @RequestBody Map<String,Object> infos){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        ScholarshipRequest scholarshipRequest = srr.findById((Integer) infos.get("id"));

        User u = scholarshipRequest.getUser();

        //If the request is already accepted we just return ok (otherwise we would duplicate the procedure below)
        if (scholarshipRequest.getState() == RequestState.Accepted){
            return new ResponseEntity<>(HttpStatus.OK);
        }

        if (infos.get("state").equals("Accepted")){
            scholarshipRequest.setState(RequestState.Accepted);
            scholarshipRequest.setAmount((int) infos.get("amount"));

            //We then deduce then amount from the minerval
            ArrayList<Minerval> minerval = minervalRepository.getMinervalsByStudentRegNoOrderByYearDesc(u.getRegNo());
            minerval.get(0).setPaidAmount(minerval.get(0).getPaidAmount() + scholarshipRequest.getAmount());
            minerval.get(0).setToPay(minerval.get(0).getToPay() - scholarshipRequest.getAmount());
            minervalRepository.save(minerval.get(0));
        }else{
            scholarshipRequest.setState(RequestState.Refused);
        }

        srr.save(scholarshipRequest);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping(value = "/scholarshipreq/{id}")
    public ResponseEntity<ScholarshipRequest> getScholReqbyId(@RequestHeader("Authorization") String token, @PathVariable long id){
        if (authServ.isNotIn(new Role[]{Role.Admin, Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        ScholarshipRequest toReturn = srr.findById(id);
        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }

    @GetMapping(value = "/unregister")
    public ResponseEntity<ArrayList<UnregisterRequest>> getAllUnregReq(@RequestHeader("Authorization") String token){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        ArrayList<UnregisterRequest> toReturn = new ArrayList<>();
        unregisterRequestRepository.findAll().forEach(toReturn::add);
        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }

    @GetMapping(value = "/unregister/{id}")
    public ResponseEntity<UnregisterRequest> getUnregbyId(@RequestHeader("Authorization") String token, @PathVariable long id){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        UnregisterRequest unregisterRequest = unregisterRequestRepository.findById(id);
        return new ResponseEntity<>(unregisterRequest, HttpStatus.OK);
    }

    @PatchMapping(value = "/unregister/{id}/{newstate}")
    public ResponseEntity<String> pathUnregReq(@RequestHeader("Authorization") String token, @PathVariable long id, @PathVariable RequestState newstate){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        UnregisterRequest unregisterRequest = unregisterRequestRepository.findById(id);
        User u = userRepository.findById(unregisterRequest.getRegNo());

        //If the request is already accepted we just return ok (otherwise we would duplicate the procedure below)
        if (unregisterRequest.getState() == RequestState.Accepted){
            return new ResponseEntity<>(HttpStatus.OK);
        }

        unregisterRequest.setState(newstate);
        unregisterRequestRepository.save(unregisterRequest);
        if (newstate == RequestState.Accepted){
            if (unregisterRequest.getCurriculum() == null){
                ArrayList<UserCurriculum> userCurricula = userCurriculumRepository.findByUserOrderByCurriculum(u);
                for (int i = 0; i < userCurricula.size(); i++){
                    userCurricula.get(i).setActual(false);
                }
                userCurriculumRepository.saveAll(userCurricula);
            }else{
                //This usercurriculum will contain the usercurriculum to set false
                UserCurriculum userCurriculum = userCurriculumRepository.findByUserAndCurriculumAndActual(u, unregisterRequest.getCurriculum(), true);
                userCurriculum.setActual(false);
                userCurriculumRepository.save(userCurriculum);
            }
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    //We look in the usercursus table if the student has already the previous year of a curriculum
    public boolean studentHasPrevYear(Curriculum curriculum, User user){
        ArrayList<UserCurriculum> userCurrList = userCurriculumRepository.findByUserOrderByCurriculum(user);
        for (int i = 0; i < userCurrList.size(); i++){
            if (userCurrList.get(i).getCurriculum().getOption().equals(curriculum.getOption()) && userCurrList.get(i).getCurriculum().getYear() == curriculum.getYear()-1){
                return true;
            }
        }
        return false;
    }
    @PostMapping("/changecurriculumreq")
    public ResponseEntity<String> addChangeCurrReq(@RequestBody Map<String,Object> reqInfos){
        User user = userRepository.findById((Integer) reqInfos.get("userId"));

        Curriculum actualCurriculum;

        //If null then it means we are in a supplementary cursus case
        if (reqInfos.get("actualcursus") == null){
            actualCurriculum = null;
        }else{
            actualCurriculum = curriculumRepository.findById((Integer) reqInfos.get("actualcursus"));
        }

        Curriculum destinationCurriculum = curriculumRepository.findById((Integer) reqInfos.get("newcursus"));

        ChangeCurriculumRequest changeCurriculumRequest = new ChangeCurriculumRequest(user, actualCurriculum, destinationCurriculum, new Date(), RequestState.Pending, RequestState.Unrequired);

        //Si l'année du cursus est plus grande que 1 et que l'étudiant n'a pas dans sa liste de cursus l'année d'en dessous alors on demande l'accord du prof
        if (destinationCurriculum.getYear() > 1 && !studentHasPrevYear(destinationCurriculum, user)){
            changeCurriculumRequest.setTeacherApprovalState(RequestState.Pending);
        }
        changeCurriculumRequestRepository.save(changeCurriculumRequest);

        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping("/changecurriculumreq")
    public  ResponseEntity<ArrayList <ChangeCurriculumRequest>> getAllChangeCurrReq(@RequestHeader("Authorization") String token){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService, Role.Teacher},token))
            return new UnauthorizedResponse<>(null);

        ArrayList<ChangeCurriculumRequest> toReturn = new ArrayList<>();

        changeCurriculumRequestRepository.findAll().forEach(toReturn::add);

        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }

    @GetMapping("/changecurriculumreq/{id}")
    public ResponseEntity<ChangeCurriculumRequest> getCCrbyId(@RequestHeader("Authorization") String token, @PathVariable long id){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Teacher,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        ChangeCurriculumRequest toReturn = changeCurriculumRequestRepository.findById(id);
        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }

    @PatchMapping("/changecurriculumreq/{id}/{newState}")
    public ResponseEntity<String> editCCReq(@RequestHeader("Authorization") String token, @PathVariable long id, @PathVariable RequestState newState){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        ChangeCurriculumRequest toEdit = changeCurriculumRequestRepository.findById(id);

        //If the request is already accepted we just return ok (otherwise we would duplicate the procedure below)
        if (toEdit.getState() == RequestState.Accepted){
            return new ResponseEntity<>(HttpStatus.OK);
        }

        toEdit.setState(newState);
        changeCurriculumRequestRepository.save(toEdit);
        if (newState == RequestState.Accepted && (toEdit.getTeacherApprovalState() == RequestState.Accepted || toEdit.getTeacherApprovalState() == RequestState.Unrequired)){
            //If actual curriculum is not null then we need to set that the user doesn't follow it anymore
            acceptProcedure(toEdit);
        }

        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PatchMapping("/changecurriculumreqteacher/{id}/{newteacherstate}")
    public ResponseEntity<String> editCCReqTeacherState(@RequestHeader("Authorization") String token, @PathVariable long id, @PathVariable RequestState newteacherstate){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Teacher},token))
            return new UnauthorizedResponse<>(null);

        ChangeCurriculumRequest toEdit = changeCurriculumRequestRepository.findById(id);

        //If the request is already accepted we just return ok (otherwise we would duplicate the procedure below)
        if (toEdit.getTeacherApprovalState() == RequestState.Accepted){
            return new ResponseEntity<>(HttpStatus.OK);
        }

        toEdit.setState(newteacherstate);
        changeCurriculumRequestRepository.save(toEdit);

        if (newteacherstate == RequestState.Accepted && toEdit.getState() == RequestState.Accepted){
            //If actual curriculum is not null then we need to set that the user doesn't follow it anymore
            acceptProcedure(toEdit);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    private void acceptProcedure(ChangeCurriculumRequest toEdit) {
        User u = toEdit.getUser();
        if (toEdit.getActualCurriculum() != null){
            ArrayList<UserCurriculum> listcurr = userCurriculumRepository.findByUserOrderByCurriculum(u);

            for (int i = 0; i < listcurr.size(); i++){
                if (listcurr.get(i).getCurriculum() == toEdit.getActualCurriculum()){
                    listcurr.get(i).setActual(false);
                }
            }

            userCurriculumRepository.saveAll(listcurr);
        }

        Calendar c = Calendar.getInstance();
        UserCurriculum userCurriculum = new UserCurriculum(u, toEdit.getDestinationCurriculum(), c.get(Calendar.YEAR), true);
        userCurriculumRepository.save(userCurriculum);
    }

    @GetMapping("/exemptionreq/{userId}")
    public ResponseEntity<ArrayList<ExemptionsRequest>> getExReqByuser(@RequestHeader("Authorization") String token, @PathVariable long userId){
        User currentUser = tokenService.getUserFromToken(token);

        //Only admin, teacher, secretary and the student himself can access a student's data here
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Teacher, Role.Secretary},token) && currentUser.getRegNo() != userId)
            return new UnauthorizedResponse<>(null);

        User u = userRepository.findById(userId);

        ArrayList<ExemptionsRequest> exList = err.findByUser(u);
        return new ResponseEntity<>(exList, HttpStatus.OK);
    }
}
