package ovh.herisson.Clyde.EndPoints.Msg;

/******************************************************
 * @file MessagesController.java
 * @author Anthony Debucquoy
 * @scope Extension messagerie
 *
 * Entry point for the messages application
 ******************************************************/

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RestController;

import lombok.AllArgsConstructor;
import ovh.herisson.Clyde.DTO.Msg.DiscussionDTO;
import ovh.herisson.Clyde.Repositories.UserRepository;
import ovh.herisson.Clyde.Repositories.Msg.DiscussionRepository;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Services.UserService;
import ovh.herisson.Clyde.Services.Msg.DiscussionService;
import ovh.herisson.Clyde.Tables.User;
import ovh.herisson.Clyde.Tables.Msg.Discussion;
import ovh.herisson.Clyde.Tables.Msg.Message;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
@AllArgsConstructor
public class MessagesController {

	private AuthenticatorService authServ;
	private DiscussionService discServ;
	private DiscussionRepository discRepo;
	private UserService userServ;

	@GetMapping("/discussions")
	public ResponseEntity<Iterable<Discussion>> getDiscussions(@RequestHeader("Authorization") String token ){
		User user = authServ.getUserFromToken(token);
		if(user == null){
			return new UnauthorizedResponse<>(null);
		}

		Iterable<Discussion> mock = discServ.getOwned(authServ.getUserFromToken(token));

		return new ResponseEntity<>(mock, HttpStatus.OK); 
	} 

	@GetMapping("/discussion/{id}")
	public ResponseEntity<DiscussionDTO> getDiscussion(@RequestHeader("Authorization") String token, @PathVariable long id){
		User user = authServ.getUserFromToken(token);
		if(user == null || !discServ.hasDiscussion(user, id) ){
			return new UnauthorizedResponse<>(null);
		}
		return new ResponseEntity<>(DiscussionDTO.construct(discRepo.findById(id).orElse(null), authServ.getUserFromToken(token)), HttpStatus.OK);
	} 

	@PatchMapping("/discussion/{id}")
	public ResponseEntity<Discussion> AlterDiscussion(@RequestHeader("Authorization") String token, @PathVariable long id, @RequestBody Discussion data){
		User user = authServ.getUserFromToken(token);
		if(user == null){
			return new UnauthorizedResponse<>(null);
		}

		Discussion disc = discRepo.findById(id).orElse(null);
		disc.setName(data.getName());
		discRepo.save(disc);
		return new ResponseEntity<>(disc, HttpStatus.OK);
	}

	@PatchMapping("/discussion/{id}/add")
	public ResponseEntity<Discussion> invite(@RequestHeader("Authorization") String token, @PathVariable long id, @RequestBody User data){
		User user = authServ.getUserFromToken(token);
		if(user == null){
			return new UnauthorizedResponse<>(null);
		}

		Discussion disc = discRepo.findById(id).orElse(null);
		User invited = userServ.getUserById(data.getRegNo());
		disc.addMember(invited);
		discRepo.save(disc);
		return new ResponseEntity<>(disc, HttpStatus.OK);
	}

	@PatchMapping("/discussion/{id}/remove")
	public ResponseEntity<Discussion> removeMember(@RequestHeader("Authorization") String token, @PathVariable long id, @RequestBody User data){
		User user = authServ.getUserFromToken(token);
		if(user == null){
			return new UnauthorizedResponse<>(null);
		}

		Discussion disc = discRepo.findById(id).orElse(null);
		User member = userServ.getUserById(data.getRegNo());
		disc.delMember(member);
		discRepo.save(disc);
		return new ResponseEntity<>(disc, HttpStatus.OK);
	}

	@PostMapping("/discussion/{id}")
	public ResponseEntity<Discussion> sendMessage(@RequestHeader("Authorization") String token, @PathVariable long id, @RequestBody Message msg){
		User user = authServ.getUserFromToken(token);
		if(user == null){
			return new UnauthorizedResponse<>(null);
		}

		Discussion disc = discRepo.findById(id).orElse(null);
		msg.setAuthor(user);
		if(disc != null)
			discServ.CreateMessage(disc, msg);
		return new ResponseEntity<>(disc, HttpStatus.OK);
	}


	@PostMapping("/discussion")
	public ResponseEntity<Discussion> createDiscussion(@RequestHeader("Authorization") String token, @RequestBody Discussion data){
		return new ResponseEntity<>(discServ.create(data.getName(), authServ.getUserFromToken(token)), HttpStatus.OK);
	}
}
