package ovh.herisson.Clyde.EndPoints.ScientificPublications;

/******************************************************
 * @file ResearcherController.java
 * @author Bartha Maxime
 * @scope Publications Scientifiques
 *
 * API class for the researchers
 ******************************************************/
import lombok.AllArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.DTO.ScientificPublications.ResearcherDTO;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Services.ScientificPublications.ResearchesService;
import ovh.herisson.Clyde.Tables.Role;
import ovh.herisson.Clyde.Tables.ScientificPublications.Researcher;

import java.util.ArrayList;
import java.util.Map;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
@AllArgsConstructor
public class ResearcherController {

    ResearchesService researchesServ;
    AuthenticatorService authServ;


    @GetMapping("/researcher/{id}")
    public ResponseEntity<ResearcherDTO> getResearcher(@PathVariable long id){
        Researcher researcher = researchesServ.getResearcherById(id);
        return new ResponseEntity<>(ResearcherDTO.construct(researcher),HttpStatus.OK);
    }

    /**
     * Everyone can access every researcher Account
     * @return all the researchers accounts
     */
    @GetMapping("/researchers")
    public ResponseEntity<Iterable<ResearcherDTO>> getAllResearchers(){
        Iterable<Researcher> researchers = researchesServ.getAllResearchers();
        ArrayList<ResearcherDTO> toReturnResearchersDTO = new ArrayList<>();
        for (Researcher researcher: researchers){
            toReturnResearchersDTO.add(ResearcherDTO.construct(researcher));
        }
        return new ResponseEntity<>(toReturnResearchersDTO, HttpStatus.OK);
    }

    @GetMapping("/researcher")
    public ResponseEntity<ResearcherDTO> getSelf(@RequestHeader("Authorization") String token){

        Researcher self = researchesServ.getResearcherByUser(authServ.getUserFromToken(token));

        if (self ==null) return new UnauthorizedResponse<>(null);

        return new ResponseEntity<>(ResearcherDTO.construct(self), HttpStatus.OK);
    }

    @PostMapping("/researcher")
    public ResponseEntity<ResearcherDTO> postResearcher(@RequestHeader("Authorization") String token, @RequestBody Researcher researcher){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Secretary}, token)){
            return new UnauthorizedResponse<>(null);
        }

        Researcher posted = researchesServ.saveResearcher(researcher);

        if (posted == null) return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);

        return new ResponseEntity<>(ResearcherDTO.construct(posted), HttpStatus.CREATED);
    }

    @PatchMapping("/researcher/{id}")
    public ResponseEntity<ResearcherDTO> patchResearcher(@RequestHeader("Authorization") String token,
                                                         @PathVariable long id,
                                                         @RequestBody Map<String ,Object> updates){

        Researcher researcher = researchesServ.getResearcherById(id);
        if (authServ.isNotIn(new Role[]{Role.Secretary,Role.Admin}, token)
                && researcher.getId() != researchesServ.getResearcherByUser(authServ.getUserFromToken(token)).getId())
            return new UnauthorizedResponse<>(null);


        if (researcher == null) return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        researchesServ.modifyResearcherData(researcher,updates);

        return new ResponseEntity<>(ResearcherDTO.construct(researcher),HttpStatus.OK);
    }

    @DeleteMapping("/researcher/{id}")
    public ResponseEntity<String> deleteResearcher(@RequestHeader ("Authorization") String token, @PathVariable long id){
       if (authServ.isNotIn(new Role[]{Role.Admin,Role.Secretary},token))
           return new UnauthorizedResponse<>(null);

       researchesServ.deleteResearcher(researchesServ.getResearcherById(id));
       return new ResponseEntity<>(HttpStatus.OK);
    }
}
