package ovh.herisson.Clyde.EndPoints.ScientificPublications;

/******************************************************
 * @file StatController.java
 * @author Bartha Maxime
 * @scope Publications Scientifiques
 *
 * Api class for handling statistics
 ******************************************************/
import lombok.AllArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Services.ScientificPublications.ResearchesService;
import ovh.herisson.Clyde.Services.ScientificPublications.StatisticsService;
import ovh.herisson.Clyde.Tables.ScientificPublications.Researcher;

import java.util.Map;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
@AllArgsConstructor
public class StatController {


    private StatisticsService statServ;
    private ResearchesService researchesServ;

    /** returns all the statistics in a 2D array
     *
     * @param id the researcher's id
     * @return all the stats in a 2D array
     */
    @GetMapping("/stats/{id}")
    public ResponseEntity<Iterable<Iterable<Map<String, Integer>>>> getStat(@PathVariable Long id){

        Researcher researcher = researchesServ.getResearcherById(id);
        if (researcher == null) return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        Iterable<Iterable<Map<String,Integer>>> stats = statServ.generateStats(researcher);

        if (stats == null) return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        return new ResponseEntity<>(stats,HttpStatus.OK);

    }
}
