package ovh.herisson.Clyde.Services.Inscription;

import org.springframework.stereotype.Service;
import ovh.herisson.Clyde.Repositories.*;
import ovh.herisson.Clyde.Repositories.Inscription.ExternalCurriculumRepository;
import ovh.herisson.Clyde.Repositories.Inscription.InscriptionRepository;
import ovh.herisson.Clyde.Repositories.Inscription.MinervalRepository;
import ovh.herisson.Clyde.Services.UserService;
import ovh.herisson.Clyde.Tables.*;
import ovh.herisson.Clyde.Tables.Inscription.ExternalCurriculum;
import ovh.herisson.Clyde.Tables.Inscription.InscriptionRequest;
import ovh.herisson.Clyde.Tables.Inscription.Minerval;

import java.util.ArrayList;
import java.util.Calendar;

@Service
public class InscriptionService {

    private final InscriptionRepository inscriptionRepo;

    private final UserRepository userRepo;

    private final UserCurriculumRepository userCurriculumRepo;

    private final CurriculumRepository curriculumRepo;

    private final MinervalRepository minervalRepository;
    private final ExternalCurriculumRepository externalCurriculumRepository;
    private final UserService userService;

    public InscriptionService(InscriptionRepository inscriptionRepo, UserRepository userRepo, UserCurriculumRepository userCurriculumRepo, CurriculumRepository curriculumRepo, MinervalRepository minervalRepository, ExternalCurriculumRepository externalCurriculumRepository, UserService userService){
        this.inscriptionRepo = inscriptionRepo;
        this.userRepo = userRepo;
        this.userCurriculumRepo = userCurriculumRepo;
        this.curriculumRepo = curriculumRepo;
        this.minervalRepository = minervalRepository;
        this.externalCurriculumRepository = externalCurriculumRepository;
        this.userService = userService;
    }

    public InscriptionRequest save(InscriptionRequest inscriptionRequest){
        return inscriptionRepo.save(inscriptionRequest);
    }

    public InscriptionRequest getById(long id){
        return inscriptionRepo.findById(id);
    }

    public Iterable<InscriptionRequest> getAll(){
        return inscriptionRepo.findAll();
    }

    public boolean modifyState(long id, RequestState requestState) {
        InscriptionRequest inscrRequest = getById(id);

        if (inscrRequest == null)
            return false;

        //If the request is already accepted we just return ok (otherwise we would duplicate the procedure below)
        if (inscrRequest.getState() == RequestState.Accepted){
            return true;
        }

        inscrRequest.setState(requestState);
        save(inscrRequest);

        //saves the user from the request if accepted from teacher and inscription services
        if (requestState == RequestState.Accepted && (inscrRequest.getEquivalenceState() == RequestState.Accepted || inscrRequest.getEquivalenceState() == RequestState.Unrequired))
        {
            return createUser(inscrRequest);
        }
        return true;
    }

    public boolean createUser(InscriptionRequest inscrRequest){
        //We must send an email here

        if (curriculumRepo.findById(inscrRequest.getCurriculumId()) == null)
            return false;

        User userFromRequest = new User(
                inscrRequest.getLastName(),
                inscrRequest.getFirstName(),
                inscrRequest.getEmail(),
                inscrRequest.getAddress(),
                inscrRequest.getCountry(),
                inscrRequest.getBirthDate(),
                inscrRequest.getProfilePicture(),
                inscrRequest.getPassword(),
                inscrRequest.getIdentityCard()
        );

        userService.save(userFromRequest);
        Calendar c = Calendar.getInstance();
        userCurriculumRepo.save(new UserCurriculum(userFromRequest, curriculumRepo.findById(inscrRequest.getCurriculumId()),c.get(Calendar.YEAR), true));

        //Create a minerval for the new student
        Minerval minerval = new Minerval(userFromRequest.getRegNo(), 0, 852, 2023);
        minervalRepository.save(minerval);

        //Assign the externals curriculums from the inscription request to newly created student
        ArrayList<ExternalCurriculum> extCurrList = externalCurriculumRepository.getExternalCurriculumByInscriptionRequest(inscrRequest);
        for (int i = 0; i < extCurrList.size(); i++){
            extCurrList.get(i).setUser(userFromRequest);
            externalCurriculumRepository.save(extCurrList.get(i));
        }
        return true;
    }
    public void delete(InscriptionRequest toDelete) {
        inscriptionRepo.delete(toDelete);
    }
}
