package ovh.herisson.Clyde.Services;

/******************************************************
 * @file LeessonRequestService.java
 * @author William Karpinski
 * @scope Extension Horaire
 ******************************************************/
import org.springframework.stereotype.Service;
import ovh.herisson.Clyde.Repositories.*;
import ovh.herisson.Clyde.Tables.*;

import java.util.Map;

@Service
public class LessonRequestService {
    private final LessonChangesRequestRepository lessonChangesRepo;

    private final UserRepository userRepo;

    private final LessonRepository lessonRepo;

    private final LessonService lessonServ;

    private final ScheduleLessonRepository scheduleLessonRepo;

    private final ScheduleRepository scheduleRepository;

    private final ScheduleLessonService scheduleLessonService;

    private final UserService userServ;

    private final CourseRepository courseRepository;
    public LessonRequestService(LessonChangesRequestRepository lessonChangesRepo,
                                UserRepository userRepo, LessonRepository lessonRepo,
                                LessonService lessonServ, ScheduleLessonRepository scheduleLessonRepo,
                                ScheduleRepository scheduleRepository, ScheduleLessonService scheduleLessonService,
                                UserService userServ, CourseRepository courseRepository) {
        this.lessonChangesRepo = lessonChangesRepo;
        this.userRepo = userRepo;
        this.lessonRepo = lessonRepo;
        this.lessonServ = lessonServ;
        this.scheduleLessonRepo = scheduleLessonRepo;
        this.scheduleRepository = scheduleRepository;
        this.scheduleLessonService = scheduleLessonService;
        this.userServ = userServ;
        this.courseRepository = courseRepository;
    }

    /**
     * Create a new lesson request
     */
    public LessonChangesRequest save(LessonChangesRequest lessonRequest){
        return lessonChangesRepo.save(lessonRequest);
    }

    /**
     * Find all the requests made by a user
     */
    public Iterable<LessonChangesRequest> findOwnRequests(User user){
        return lessonChangesRepo.findOwnRequests(user);
    }
    public LessonChangesRequest findById(long id){
        return lessonChangesRepo.findById(id);
    }
    /**
     *  Return all the requests
     */
    public Iterable<LessonChangesRequest> getAll(){return lessonChangesRepo.findAll();}

    /**
     * Create a lesson if a request is accepted
     */
    public boolean modifyCreateRequestState(LessonChangesRequest lessonRequest, RequestState state, String local ){
        if(lessonRequest == null || state == lessonRequest.getState()  || state == null){
            return false;}
        if (state == RequestState.Accepted){
            Course course = courseRepository.findById(lessonRequest.getCourse().getCourseID());
            if(courseRepository.findById(lessonRequest.getCourse().getCourseID())==null|| local == null){
                return false;}
            Lesson lesson = new Lesson();
            lesson.setCourse(course);
            lesson.setLessonStart(lessonRequest.getLessonStart());
            lesson.setLessonEnd(lessonRequest.getLessonEnd());
            lesson.setColor(lessonRequest.getColor());
            lesson.setLocal(local);
            lesson.setLessonType(lessonRequest.getLessonType());

            lesson = lessonRepo.save(lesson);
            scheduleLessonService.saveToAllSchedule(lesson);
            }
        lessonRequest.setState(state);
        save(lessonRequest);
        return true;
    }

    public Iterable<LessonChangesRequest> findRequestByLessonId(long id){
      return lessonChangesRepo.findRequestByLessonId(id);
    }

    /**
     *  Refuse all the lesson request that depends on a certain lesson
     *  Used after the deletion of the lesson
     */
    public void refuseAllByLessonId(long id){
      Iterable<LessonChangesRequest> toRefuse = findRequestByLessonId(id);
      for(LessonChangesRequest element : toRefuse)
        element.setState(RequestState.Refused);
        
  }

    /**
     *  Modify a lesson if a request is accepted
     */
    public boolean modifyChangeRequestState(Map<String, Object> updates, long lessonId,RequestState state){
        if(state == RequestState.Accepted){
            Lesson lesson = lessonServ.findById(lessonId);
            return lessonServ.modifyData(lesson.getLessonID(),updates);
        }
        return true;
    }

    /**
     *  Delete a lesson if a request is accepted
     */
    public void modifyDeleteRequest(LessonChangesRequest lessonChangesRequest, RequestState state){
        if(state == RequestState.Accepted){
            lessonServ.delete(lessonServ.findById(lessonChangesRequest.getLessonId()));
            refuseAllByLessonId(lessonChangesRequest.getLessonId());
            }
    }

    /**
     * Construct a lesson request
     */
    public LessonChangesRequest createLessonRequest(Map<String,Object> lessonInfos) {
        LessonChangesRequest target = new LessonChangesRequest();

        for (Map.Entry<String, Object> entry : lessonInfos.entrySet()) {
            System.out.println(entry.toString());
            if(entry.getValue() != null){
                switch (entry.getKey()) {
                    case "requestType":
                    target.setRequestType((int) entry.getValue());
                    break;
                    case "lessonStart":
                        target.setLessonStart((String) entry.getValue());
                        break;
                    case "lessonEnd":
                        target.setLessonEnd((String) entry.getValue());
                        break;
                    case "color":
                        target.setColor((String) entry.getValue());
                        break;
                    case "user":
                        target.setUser(userServ.getUserById((int) entry.getValue()));
                        break;
                    case "lessonType":
                        target.setLessonType((String) entry.getValue());
                        break;
                    case "course":
                        target.setCourse(courseRepository.findById((int) entry.getValue()));
                        break;
                    case "lessonId":
                        target.setLessonId((int) entry.getValue());
                        break;

                }
            }
        }
        target.setState(RequestState.Pending);
        return target;
    }

    public void delete (LessonChangesRequest toDelete) {
        lessonChangesRepo.delete(toDelete);
    }
}
