package ovh.herisson.Clyde.Services;


/******************************************************
 * @file LessonService.java
 * @author William Karpinski
 * @scope Extension Horaire
 ******************************************************/
import org.springframework.stereotype.Service;
import ovh.herisson.Clyde.Repositories.*;
import ovh.herisson.Clyde.Tables.*;

import java.util.ArrayList;
import java.util.Map;

@Service
public class LessonService {
    private final LessonRepository lessonRepo;

    private final ScheduleLessonRepository scheduleLessonRepo;
    private final UserCurriculumRepository userCurriculumRepo;
    private final UserService userServ;
    private final CourseRepository courseRepo;
    private final CurriculumCourseRepository curriculumCourseRepo;
    public LessonService(LessonRepository lessonRepo, ScheduleLessonRepository scheduleLessonRepo, UserCurriculumRepository userCurriculumRepo, UserService userServ, CourseRepository courseRepo, CurriculumCourseRepository curriculumCourseRepo){
        this.lessonRepo = lessonRepo;
        this.scheduleLessonRepo = scheduleLessonRepo;
        this.userCurriculumRepo = userCurriculumRepo;
        this.userServ = userServ;
        this.courseRepo = courseRepo;
        this.curriculumCourseRepo = curriculumCourseRepo;
    }

    /**
     * Create a lesson
     */
    public Lesson save(Lesson lesson){
        return lessonRepo.save(lesson);
    }
    /**
     * Find a lesson by its id
     */
    public Lesson findById(long id){
        return lessonRepo.findById(id);
    }
    /**
     * Return all the lessons
     */
    public Iterable<Lesson> findAll(){return lessonRepo.findAll();}
    /**
     * Return all a teacher's lessons
     */
    public Iterable<Lesson> findAllOwnedLesson(User teacher){
        ArrayList<Lesson> toReturn = new ArrayList<>();
        ArrayList<Course> coursesOwned = (ArrayList<Course>) courseRepo.findAllOwnedCoures(teacher);
        for (Course element : coursesOwned) {
            for(Lesson lesson : lessonRepo.findLessonByCourse(element))
                toReturn.add(lesson);
        }
        return toReturn;
    }
    /**
     * Return all a student's lessons
     */
    
  public Iterable<Lesson> findOnesLessons(User student){
    ArrayList<Lesson> toReturn = new ArrayList<>();
    ArrayList<Course> courses = new ArrayList<>();
    ArrayList<UserCurriculum> userCurricula = userCurriculumRepo.findByUserAndActual(student, true);
      for (UserCurriculum userCurriculum : userCurricula) {
          curriculumCourseRepo.findCoursesByCurriculum(userCurriculum.getCurriculum()).forEach((item) -> {
              //We need this to eliminate clones because a course can belong to several curriculums
              if (!courses.contains(item)) {
                  System.out.println(item.getTitle());
                  courses.add(item);
              }
          });
      }
    for (Course element : courses) {
          for(Lesson lesson : lessonRepo.findLessonByCourse(element))
              toReturn.add(lesson);
      }

    return toReturn;
    }
    /**
     *  Construct a new lesson
     */
    public Lesson createLesson(Map<String,Object> lessonInfos) {
        Lesson target = new Lesson();

        for (Map.Entry<String, Object> entry : lessonInfos.entrySet()) {
            switch (entry.getKey()) {
                case "lessonStart":
                    target.setLessonStart((String) entry.getValue());
                    break;
                case "lessonEnd":
                    target.setLessonEnd((String) entry.getValue());
                case "color":
                    target.setColor((String) entry.getValue());
                    break;
                case "local":
                    target.setLocal((String) entry.getValue());
                    break;
                case "lessonType":
                    target.setLessonType((String) entry.getValue());
                    break;
                case "courseID":
                    target.setCourse(courseRepo.findById((int) entry.getValue()));
                    break;
            }

        }
        return target;
    }
    /**
     * Modify a lesson
     */
    public boolean modifyData(long id, Map<String ,Object> updates){
        Lesson target = lessonRepo.findById(id);

        if(target == null)
            return false;

        for (Map.Entry<String , Object> entry: updates.entrySet()){
            switch (entry.getKey()){
                case "lessonStart":
                    target.setLessonStart((String) entry.getValue());
                    break;
                case "lessonEnd":
                    target.setLessonEnd((String) entry.getValue());
                    break;            
                case "local":
                    target.setLocal((String) entry.getValue());
                    break;
                case "lessonType":
                    target.setLessonType((String) entry.getValue());
                    break;
            }
        }
        Lesson lesson = lessonRepo.save(target);
        ScheduleLesson scheduleLesson = scheduleLessonRepo.findByLesson(lesson);
        Iterable<User> users = userCurriculumRepo.findUsersByCurriculum(scheduleLesson.getSchedule().getCurriculum());
        for(User user: users){
            userServ.Notify(user, new Notification("Course modified in the schedule", "Course Modified " + lesson.getCourse().getTitle() , "/#/schedule"));
        }
        return true;
    }
    /**
     * Delete a lesson
     */
    public void delete(Lesson lesson){
        lessonRepo.delete(lesson);
    }


}
