package ovh.herisson.Clyde.Services.Msg;

import java.util.List;

/******************************************************
 * @file DiscussionService.java
 * @author Anthony Debucquoy
 * @scope Extension messagerie
 *
 * Various function utilised by the messages application
 ******************************************************/


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.util.JSONPObject;

import ovh.herisson.Clyde.Repositories.Msg.DiscussionRepository;
import ovh.herisson.Clyde.Services.UserService;
import ovh.herisson.Clyde.Tables.Notification;
import ovh.herisson.Clyde.Tables.User;
import ovh.herisson.Clyde.Tables.Msg.Discussion;
import ovh.herisson.Clyde.Tables.Msg.Message;

@Service
public class DiscussionService {

	@Autowired
	private DiscussionRepository discRepo;
	@Autowired
	private UserService userServ;

	public Discussion create(String name, User author){
		return discRepo.save(new Discussion(name, author));
	}

	/**
	 * list discussions owned by a certain user
	 */
	public Iterable<Discussion> getOwned(User author){
		return discRepo.findByMembership(author.getRegNo());
	}

	/**
	 * Create a message and link it to it's discussion
	 */
	public Discussion CreateMessage(Discussion disc, Message msg){
		for(User u: disc.getMembers()){
			userServ.Notify(u, new Notification("msg.notification.new", msg.getContent(), "/#/msg"));
		}
		disc.addMessage(msg);
		return discRepo.save(disc);
	}

	/**
	 * Check if a user is in a discussion
	 */
    public boolean hasDiscussion(User user, long id) {
		Discussion disc = discRepo.findById(id).orElse(null);
		List<User> members = disc.getMembers();
		return members.contains(user);
    }
}
