package ovh.herisson.Clyde.Tables.Msg;

/******************************************************
 * @file Discussion.java
 * @author Anthony Debucquoy
 * @scope Extension messagerie
 *
 * Discussion allow to regroupe multiple user in and message together
 * for the messages application to work
 ******************************************************/


import java.util.List;

import jakarta.persistence.CascadeType;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.JoinTable;
import jakarta.persistence.ManyToMany;
import jakarta.persistence.OneToMany;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import ovh.herisson.Clyde.Tables.User;

@Entity
@Getter
@Setter
@NoArgsConstructor
public class Discussion{

	@Id
    @GeneratedValue(strategy = GenerationType.AUTO)
	private long id;
	private String name;

    @ManyToMany
	@JoinTable(
		name = "discussion_members",
		joinColumns = @JoinColumn(name = "discussion_id"),
		inverseJoinColumns = @JoinColumn(name = "user_id")
	)
	private List<User> members;

	@OneToMany(mappedBy="discussion", orphanRemoval = true, cascade = CascadeType.ALL)
	private List<Message> msgs;

	public Discussion(String name){
		this.name = name;
	}

	public Discussion(String name, User user){
		this.name = name;
		this.members = List.of(user);
	}

	public void addMessage(Message msg){
		msg.setDiscussion(this);
		msgs.add(msg);
	}

    public void addMember(User user) {
		members.add(user);
    }

    public void delMember(User user) {
		members.remove(user);
    }
}
