package ovh.herisson.Clyde.Tables;

import com.fasterxml.jackson.annotation.JsonIgnore;
import jakarta.persistence.*;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.hibernate.annotations.GenericGenerator;
import ovh.herisson.Clyde.Tables.Msg.Discussion;
import ovh.herisson.Clyde.Tables.Msg.Message;
import ovh.herisson.Clyde.Tables.Notification.Status;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@Table(name = "Users")
@NoArgsConstructor
@Data
public class User {
    @Id
    @GenericGenerator(name = "userGen", type = ovh.herisson.Clyde.Tables.RegNoGenerator.class)
    @GeneratedValue(generator = "userGen")
    private Long regNo;
    private String lastName;
    private String firstName;
    @Column(unique = true)
    private String email;
    private String address;
    private String country;
    private Date birthDate;
    private String profilePictureUrl;
    private Role role;
    private String identityCardUrl;
	@JsonIgnore
    private String password;

	@JsonIgnore
	@OneToMany(mappedBy = "user", cascade = CascadeType.ALL)
	private List<Notification> notifications = new ArrayList<>();

	////// Extension Messagerie /////
	@JsonIgnore
	@OneToMany(mappedBy = "author", cascade = CascadeType.ALL)
	private List<Message> msgs;

	@JsonIgnore
	@ManyToMany( mappedBy = "members" )
	private List<Discussion> discussions;
	/////////////////////////////////

    /////////////////////////////////
    public User(String lastName, String firstName, String email, String address,
                String country, Date birthDate, String profilePictureUrl, Role role, String password)
    {
        this.lastName = lastName;
        this.firstName = firstName;
        this.email = email;
        this.address = address;
        this.country = country;
        this.birthDate = birthDate;
        this.profilePictureUrl = profilePictureUrl;
        this.role = role;
		this.password = (new BCryptPasswordEncoder()).encode(password);
    }


    //This constructor is used to add a student
    public User(String lastName, String firstName, String email, String address,
                String country, Date birthDate, String profilePictureUrl, String password,String identityCardUrl)
    {
        this.lastName = lastName;
        this.firstName = firstName;
        this.email = email;
        this.address = address;
        this.country = country;
        this.birthDate = birthDate;
        this.profilePictureUrl = profilePictureUrl;
		this.password = (new BCryptPasswordEncoder()).encode(password);
        this.role = Role.Student;
        this.identityCardUrl = identityCardUrl;
    }

	public void setPassword(String password) {
		this.password = (new BCryptPasswordEncoder()).encode(password);
	}
}
