import { restGet } from './restConsumer.js' 
import { ref, computed } from 'vue'
import i18n from '@/i18n.js'

// Liste des apps
import LoginPage from '@/Apps/Login.vue'
import Profil from "@/Apps/Profil.vue"
import Courses from "@/Apps/ManageCourses.vue"
import Users from "@/Apps/UsersList.vue"
import Students from "@/Apps/StudentsList.vue"
import Schedule from "@/Apps/Schedule.vue"
import ManageSchedule from "@/Apps/ManageSchedule.vue"
import ManageOwnedLessons from "@/Apps/ManageOwnLessons.vue";
import LessonRequests from "@/Apps/LessonRequests.vue";
import Msg from "@/Apps/Msg.vue"
import Forums from '@/Apps/Forums.vue'
import Payments from "@/Apps/Inscription/PaymentInfo.vue";
import ManageRequests from "@/Apps/Inscription/ManageRequests.vue";
import ManageResearcherProfile from "@/Apps/ScientificPublications/ManageResearcherProfile.vue";
import ListResearches from "@/Apps/ScientificPublications/ListResearches.vue";
import ResearcherProfile from "@/Apps/ScientificPublications/ResearcherProfile.vue";
import CreateUser from "@/Apps/CreateUser.vue";

const apps = {
		'/schedule': Schedule,
		'/manage-schedule': ManageSchedule,
		'/login': LoginPage,
		'/requests': ManageRequests,
		'/profil': Profil,
		'/manage-courses' : Courses,
		'/users-list' : Users,
		'/students-list' : Students,
		'/manage-researcher-profile' : ManageResearcherProfile,
		'/researches' : ListResearches,
		'/researcher-profile': ResearcherProfile,
		'/create-user': CreateUser,
		'/manage-owned-lessons': ManageOwnedLessons,
		'/manage-schedule-requests' : LessonRequests,
		'/msg' : Msg,
		'/forums': Forums,
		'/payments': Payments
}

const appsList = {
		'ListResearches': {path:'#/researches', icon:'fa-book-bookmark',text:i18n("app.list.researches")},
		'Msg': { path: '#/msg', icon: 'fa-comment', text: i18n("app.messages") },
		'Notification': { path: '#/notifs', icon: 'fa-bell', text: i18n("app.notifications") },
		'Forum': { path: '#/forums', icon: 'fa-envelope', text: i18n("app.forum") },
		'Schedule': { path: '#/schedule', icon: 'fa-calendar-days', text: i18n("app.schedules") },
		'ManageSchedules': { path: '#/manage-schedule', icon: 'fa-calendar-days', text: i18n("app.manageSchedules")},
		'ManageCourses': { path: '#/manage-courses', icon: 'fa-book', text: i18n("app.manage.courses") },
		'StudentsList':{ path: '#/students-list',icon: 'fa-users',text: i18n("app.studentList")},
		'UsersList':{ path: '#/users-list',icon: 'fa-users',text: i18n("app.users")},
		'ManageResearcherProfile':{path:'#/manage-researcher-profile',icon:'fa-book-bookmark',text:i18n("app.manage.researcherProfile")},
		'CreateUser':{path:'#/create-user',icon:'fa-plus', text:i18n("app.Create.User")},
		'ManageOwnedLessons':{path: '#/manage-owned-lessons',icon:'fa-calendar-days',text: i18n("app.manageOwnLessons")},
		'LessonRequests':{path: '#/manage-schedule-requests', icon:'fa-book', text: i18n("app.lessonRequests")},
		'Requests': { path: '#/requests', icon: 'fa-users', text: "Requests" },
		'Payments':{path: '#/payments', icon:'fa-users', text:i18n("app.payments")}
}

const currentPath = ref(window.location.hash)

export const currentView = computed(() => {
		return apps[currentPath.value.split("?")[0].slice(1) || '/']
})

/**
 * Return the list of app accesible by a logged (or not)
 * user.
 */
export async function appList(){
		let ret = [];
		let userAppList = await restGet("/apps");
		for (let userapp in userAppList) {
				if(appsList[userAppList[userapp]] != null){
						ret.push(appsList[userAppList[userapp]])
				}
		}
		return ret;
}

/**
 * Check if the specified page is authorized for the
 * user
 */
export async function checkPage(page){
		return restGet("/apps/" + page)
}

window.addEventListener('hashchange', () => {
		currentPath.value = window.location.hash
})

// vim:set noet sts=0 sw=4 ts=2 tw=2:
