/**
 * curriculum API
 */

import { restGet, restPost, restDelete, restPatch } from './restConsumer.js'

/**
 * Create a new curriculum (bundle of courses)
 * @param courses	list of courses
 */
export async function createcurriculum(courses){
	return restPost("/curriculum", {courses: courses} );
}

/**
 * Delete the specified curriculum
 */
export async function deletecurriculum(id){
	return restDelete("/curriculum/" + id);
}

export async function getAllCurriculums(){
  return restGet("/curriculums");
}

export async function getCurriculumsByCourse(id){
	return restGet("/course/curriculum/"+ id);
}
/**
 * Get informations on a particular curriculum
 *
 * @param id	identification of the curriculum
 *
 * @return list of courses 
 */
export async function getcurriculum(id){
	return restGet("/curriculum/" + id);
}

/**
 * Modify the courses of a curriculum
 *
 * @param id		the id of the curriculum
 * @param courses	list of new courses 
 */
export async function altercurriculum(id, courses){
	return restPatch("/curriculum/" + id, courses);
}


export async function getSelfCurriculum(){
  return restGet("/curriculum");
}

export async function getSomeonesCurriculumList(user){
	return restGet("/onescurriculum/"+user)
}

export async function addCourseToCurriculum(id,courseID){
	return restPost("/curriculum/"+id, courseID);
}

