import { getCookie } from '../utils.js'
import { toast } from 'vue3-toastify'

const restURL = import.meta.env.VITE_CLYDE_MODE === 'container' ? "http://localhost:8080": import.meta.env.DEV ? "http://localhost:8080" : "https://clyde.herisson.ovh/api"

export function restGet(endPoint) {
	return _rest(endPoint, {method: "GET"});
}

export function restPost(endPoint, data) {
	return _rest(endPoint, {method: "POST", credentials: 'include', body: JSON.stringify(data)});
}

export function restPostFile(endPoint, file){
	let headers = new Headers();
	return _rest(endPoint, {method: "POST", credentials: 'include', body: file, headers: headers });
}

export function restDelete(endPoint) {
	return _rest(endPoint, {method: "DELETE"});
}

export function restDeleteItem(endPoint, data){
	return _rest(endPoint, {method: "DELETE", credentials: 'include', body: JSON.stringify(data)});
}
export function restPatch(endPoint, data) {
	return _rest(endPoint, {method: "PATCH", credentials: 'include', body: JSON.stringify(data)});
}

/**
 * backbone for the request made by the frontend
 *
 * specification
 *  - If the user has "session_token" cookie set, it will use it in the authorization field of the http request
 *  - The result will be returned as a json to access fields easily ( the backend should send json response )
 *
 *  @Example _rest("/ping", {user: data}) -> {id:0, txt:"pong"}
 */
function _rest(endPoint, config){
	endPoint.at(0) != "/" ? console.error("Carefull, you certainly should put a / at the begenning of your endPoint ") : true;
	let session_token = getCookie("session_token");
	let headers = new Headers({
		'Authorization': session_token,
		'Content-Type': 'application/json',
	});
	config['headers'] = config['headers'] == null ? headers : config['headers'];

	let ret = fetch(restURL + endPoint, config);
	if(config['toast']){
		ret = toast.promise(ret,
		{
			pending: config['pending'] != null ? config['pending'] : 'pending',
			error: config['error'] != null ? config['error'] : 'Network Failure...',
			success: config['success'] != null ? config['success'] : {render(res){
				return res.data.ok ? "Success" : "error";
			}}})
	}
	return ret.then( e => e.json()).catch( e => e );
}
